//
//  PCTransaction.h
//  PayControl
//
//  Created by Roman Kolesnichenko on 22.08.16.
//  Copyright © 2016 SafeTech. All rights reserved.
//

#import "PCQRCode.h"

/** Объект транзакции. Только для чтения. Создается с помощью методов класса: transactionFromQRCode: или  transactionFromQRCodeString: */
@interface PCTransaction : NSObject

/// ****************************************************************************
/// @name Методы класса
/// ****************************************************************************

//+ (instancetype)transactionFromResponceString:(NSString*)responceString;

/** Созадание экземпляра класса из объекта QR кода 
 @param transactionQRCode объект PCTransactionQRCode
 @return Экземпляр PCTransaction
 */
+ (instancetype)transactionFromQRCode:(PCQRCode*)qrCode;

/** Созадание экземпляра класса из строки с валидными данными QR кода 
 @param transactionQRCodeString строка с данными QR кода. 
 @return Экземпляр PCTransaction или nil - если данные в переданной transactionQRCodeString строке не валидны
 */
+ (instancetype)transactionFromQRCodeString:(NSString*)transactionQRCodeString;

/// ****************************************************************************
/// @name Свойства
/// ****************************************************************************

/** Идентификатор прикладной системы */
@property(nonatomic, readonly) NSString *systemID;

/** Идентификатор пользователя */
@property(nonatomic, readonly) NSString *userID;

/** Идентификатор транзакции */
@property(nonatomic, readonly) NSString *transactionID;

/** Предпочитаемый тип подтверждения транзакции */
@property(nonatomic, readonly) PCConfirmationType confirmationType;

/** Длина кода подтверждения */
@property(nonatomic, readonly) NSUInteger codeLength;

/** Текстовая информация из данных транзакции */
@property(nonatomic, readonly) NSString *messageText;

/** Бинарная информация из данных транзакции */
@property(nonatomic, readonly) NSString *messageBinaryData;

@end
