//
//  PayControlSDK.h
//  PayControl
//
//  Created by ООО СэйфТек on 04.09.15.
//  Copyright (c) 2015 ООО СэйфТек. All rights reserved.
//

#import "PCKey.h"
#import "PCTransaction.h"
#import "PCKeyInfo.h"
#import "PCError.h"
#import "PCISError.h"
#import "PCQRCodeReader.h"
#import "PCPDFView.h"
#import "PCLogger.h"

//! Project version number for PayControlSDK.
FOUNDATION_EXPORT double PayControlSDKVersionNumber;

//! Project version string for PayControlSDK.
FOUNDATION_EXPORT const unsigned char PayControlSDKVersionString[];

/**
 PayControlSDK class
 */
@interface PayControlSDK : NSObject

/**
 Устанавливает уровень логирования в debug режиме

 @param logLevel Значение типа PCLogLevel. Значения могут комбинироваться, например PC_LOG_DEBUG | PC_LOG_KEY_VALUES выводит в лог и сообщения и значения ключей для большей информативности
 */
+ (void)setLogging:(PCLogLevel)logLevel;

/**
 Экземпляр синглтона. Все обращаения к методам и свойствам SDK должны происходить через него.
 */
+ (PayControlSDK * _Nonnull)sharedInstance;

#pragma mark - calculateConfirmationCode
/// ****************************************************************************
/// @name Методы генерации кода подтверждения
/// ****************************************************************************

/**
 Генерация кода подтверждения
 @param key Ключ для формирования кода подтверждения. Handle ключевой информации должен иметь валидное значение
 @param data Данные для подтверждения
 @param confirmationCodeLength Длина кода подтверждения (в десятичных символах)
 @param completion block, возвращает параметры в место вызова метода.
 @return error Ошибка выполнения метода. Если error = nil, то метод выполнился успешно
 @return confirmationCode Сгенерированный код для подтверждения транзакции
 @return fullConfirmationCode Сгенерированный полный код для подтверждения транзакции
 */
- (void)calculateConfirmationCode:(PCKey * _Nonnull)key
                             data:(NSString * _Nonnull)data
           confirmationCodeLength:(NSUInteger)confirmationCodeLength
                       completion:(void (^ _Nonnull)(PCError * _Nullable error, NSString * _Nullable confirmationCode, NSString * _Nullable fullConfirmationCode))completion;

/**
 Генерация кода подтверждения
 @param key Ключ для формирования кода подтверждения. Handle ключевой информации должен иметь валидное значение
 @param transaction Транзакция для подтверждения
 @param confirmationCodeLength Длина кода подтверждения (в десятичных символах)
 @param completion Блок куда возвращается результат выполнения
 @return error Ошибка выполнения метода. Если error = nil, то метод выполнился успешно
 @return confirmationCode Сгенерированный код для подтверждения транзакции
 @return fullConfirmationCode Сгенерированный полный код для подтверждения транзакции
 */
- (void)calculateConfirmationCode:(PCKey * _Nonnull)key
                      transaction:(PCTransaction * _Nonnull)transaction
           confirmationCodeLength:(NSUInteger)confirmationCodeLength
                       completion:(void (^ _Nonnull)(PCError * _Nullable error, NSString * _Nullable confirmationCode, NSString * _Nullable fullConfirmationCode))completion;

#pragma mark - calculateDeclineCode
/// ****************************************************************************
/// @name Методы генерации кода отмены
/// ****************************************************************************

/**
 Генерация кода отмены
 @param key Ключ для формирования кода отмены. Handle ключевой информации должен иметь валидное значение
 @param data Данные для отмены
 @param declineCodeLength Длина кода отмены (в десятичных символах)
 @param completion block, возвращает параметры в место вызова метода.
 @return error Ошибка выполнения метода. Если error = nil, то метод выполнился успешно
 @return declineCode Сгенерированный код для отмены транзакции
 @return fullDeclineCode Сгенерированный полный код для отмены транзакции
 */
- (void)calculateDeclineCode:(PCKey * _Nonnull)key
                        data:(NSString * _Nonnull)data
           declineCodeLength:(NSUInteger)declineCodeLength
                  completion:(void (^ _Nonnull)(PCError * _Nullable error, NSString * _Nullable declineCode, NSString * _Nullable fullDeclineCode))completion;

/**
 Генерация кода отмены
 @param key Ключ для формирования кода отмены. Handle ключевой информации должен иметь валидное значение
 @param transaction Транзакция для отмены
 @param declineCodeLength Длина кода отмены (в десятичных символах)
 @param completion Блок куда возвращается результат выполнения
 @return error Ошибка выполнения метода. Если error = nil, то метод выполнился успешно
 @return declineCode Сгенерированный код для отмены транзакции
 @return fullDeclineCode Сгенерированный полный код для отмены транзакции
 */
- (void)calculateDeclineCode:(PCKey * _Nonnull)key
                 transaction:(PCTransaction * _Nonnull)transaction
           declineCodeLength:(NSUInteger)declineCodeLength
                  completion:(void (^ _Nonnull)(PCError * _Nullable error, NSString * _Nullable declineCode, NSString * _Nullable fullDeclineCode))completion;

#pragma mark - calculateSignature
/// ****************************************************************************
/// @name Методы формирования подписи данных
/// ****************************************************************************

/**
 Формирование подписи данных
 @param key Ключ для формирования подписи. Handle ключевой информации должен иметь валидное значение
 @param data Данные для подписания
 @param completion block, возвращает параметры в место вызова метода.
 @returns error Ошибка выполнения метода. Если error = nil, то метод выполнился успешно
 @returns signature Подпись
 */
- (void)calculateSignature:(PCKey * _Nonnull)key
                      data:(NSString * _Nonnull)data
                completion:(void (^ _Nonnull)(PCError * _Nullable error, NSString * _Nullable signature))completion;

/**
 Формирование подписи данных
 @param key Ключ для формирования подписи. Handle ключевой информации должен иметь валидное значение
 @param transaction Транзакция для подписания
 @returns error Ошибка выполнения метода. Если error = nil, то метод выполнился успешно
 @returns signature Подпись
 */
- (void)calculateSignature:(PCKey * _Nonnull)key
               transaction:(PCTransaction * _Nonnull)transaction
                completion:(void (^ _Nonnull)(PCError * _Nullable error, NSString * _Nullable signature))completion;

#pragma mark - Server API methods
/// ****************************************************************************
/// @name Методы взаимодействия с сервером
/// ****************************************************************************

/** Метод для регистрации токена устройства для пушей
 @param key Экземпляр ключа
 @param deviceToken токен устройства
 @param completion Блок куда возвращается результат выполнения
 @return error Экземпляр PCISError - результат выполнения запроса, nil - если метод выполнился успешно
 */
- (void)doRegisterDevice:(PCKey * _Nonnull)key
             deviceToken:(NSString * _Nonnull)deviceToken
              completion:(void (^ _Nonnull)(PCISError * _Nullable error))completion;

/** Метод для регистрации отпечатка устройства
 @param key Экземпляр ключа
 @param fingerprint Отпечаток утстройства
 @param completion блок куда возвращается результат выполнения
 @return error Экземпляр PCISError - результат выполнения запроса, nil - если метод выполнился успешно
 */
- (void)doRegisterFingerprint:(PCKey * _Nonnull)key
                  fingerprint:(NSString * _Nonnull)fingerprint
                   completion:(void (^ _Nonnull)(PCISError * _Nullable error))completion;

/** Метод для получения данных транзакции
 @param key Экземпляр ключа
 @param transactionID Идентификатор транзакции
 @param completion Блок куда возвращается результат выполнения
 @return error Экземпляр PCISError - результат выполнения запроса, nil - если метод выполнился успешно
 @return transaction Экземпляр PCTransaction или nil если произошла ошибка
 */
- (void)doGetTransactionData:(PCKey * _Nonnull)key
               transactionID:(NSString * _Nonnull)transactionID
                  completion:(void (^ _Nonnull )(PCISError * _Nullable error, PCTransaction * _Nullable transaction))completion;

/** Метод получения списка неподтвержденных транзакций для пользователя 
 @param key Экземпляр ключа
 @param completion Блок куда возвращается результат выполнения
 @return error Экземпляр PCISError - результат выполнения запроса, nil - если метод выполнился успешно
 @return transactionsIDsArray Массив идентификаторов пользовательских транзакций или nil если произошла ошибка
 */
- (void)doGetTransactionsList:(PCKey * _Nonnull)key
               completion:(void (^ _Nonnull)(PCISError * _Nullable error, NSArray <NSString*> * _Nullable transactionsIDsArray))completion;

/** Метод для отправки кода подтверждения через IS
 @param key Экземпляр ключа
 @param transactionID Идентификатор транзакции
 @param fullConfirmationCode Код подтверждения
 @param signature Подпись, если используется ассиметричное шифрование
 @param completion блок куда возвращается результат выполнения
 @return error Экземпляр PCISError - результат выполнения запроса, nil - если метод выполнился успешно
 */
- (void)doSendConfirmationCodeByPush:(PCKey * _Nonnull)key
                       transactionID:(NSString * _Nonnull)transactionID
                fullConfirmationCode:(NSString * _Nonnull)fullConfirmationCode
                           signature:(NSString * _Nullable)signature
                          completion:(void (^ _Nonnull)(PCISError * _Nullable error))completion;

/** Метод для отправки кода подтверждения Online 
 @param key Экземпляр ключа
 @param transactionID Идентификатор транзакции
 @param fullConfirmationCode Код подтверждения
 @param signature Подпись, если используется ассиметричное шифрование
 @param completion Блок куда возвращается результат выполнения
 @return error Экземпляр PCISError - результат выполнения запроса, nil - если метод выполнился успешно
 */
- (void)doSendConfirmationCodeByUrl:(PCKey * _Nonnull)key
                      transactionID:(NSString * _Nonnull)transactionID
               fullConfirmationCode:(NSString * _Nonnull)fullConfirmationCode
                          signature:(NSString * _Nullable)signature
                         completion:(void (^ _Nonnull)(PCISError * _Nullable error))completion;

/** Метод для отправки кода отклонения через IS
 @param key Экземпляр ключа
 @param transactionID Идентификатор транзакции
 @param fullDeclineCode Код отклонения
 @param completion блок куда возвращается результат выполнения
 @return error Экземпляр PCISError - результат выполнения запроса, nil - если метод выполнился успешно
 */
- (void)doSendDeclineCodeByPush:(PCKey * _Nonnull)key
                  transactionID:(NSString * _Nonnull)transactionID
                fullDeclineCode:(NSString * _Nonnull)fullDeclineCode
                     completion:(void (^ _Nonnull)(PCISError * _Nullable error))completion;

/** Метод для отправки кода отклонения Online
 @param key Экземпляр ключа
 @param transactionID Идентификатор транзакции
 @param fullDeclineCode Код отклонения
 @param completion Блок куда возвращается результат выполнения
 @return error Экземпляр PCISError - результат выполнения запроса, nil - если метод выполнился успешно
 */
- (void)doSendDeclineCodeByUrl:(PCKey * _Nonnull)key
                 transactionID:(NSString * _Nonnull)transactionID
               fullDeclineCode:(NSString * _Nonnull)fullDeclineCode
                    completion:(void (^ _Nonnull)(PCISError * _Nullable error))completion;

/** Метод для регистрация открытого ключа на сервере
 @param key Экземпляр ключа
 @param completion Блок куда возвращается результат выполнения
 @return error Экземпляр PCISError - результат выполнения запроса, nil - если метод выполнился успешно
 */
- (void)doRegisterPublicKey:(PCKey * _Nonnull)key completion:(void (^ _Nonnull)(PCISError * _Nullable error))completion;

#pragma mark - Interaction with the keystorage
/// ****************************************************************************
/// @name Методы и свойства взаимодействия с хранилищем ключей
/// ****************************************************************************

/** Перечисление доступных наборов ключевой информации
 @param completion block, возвращает параметры в место вызова метода.
 @return array массив описаний ключевой информации (PCKeyInfo), nil - если произошла ошибка выполнения
 @return error Ошибка выполнения метода. Если error = nil, то метод выполнился успешно
 */
- (void)listKeys:(void (^ _Nonnull)(PCError * _Nullable error, NSArray <PCKeyInfo*> * _Nullable keysArray))completion;

/**
 Удаление ключа из хранилища по его идентификатору
 @param keyID Идентификатор ключа в хранилище PayControl
 @returns error ошибка. Если error = nil, то удаление завершилось успешно
 */
- (PCError * _Nullable)deleteKey:(NSString * _Nonnull)keyID;

/**
 Удаление ключа из хранилища по его идентификатору пользователя
 @param userID Идентификатор пользователя ключа в хранилище PayControl
 @returns error ошибка. Если error = nil, то удаление завершилось успешно
 */
- (PCError * _Nullable)deleteKeyWithUserID:(NSString * _Nonnull)userID;

/** Наличие ключей в хранилище
 @return YES, если ключей в хранилище нет
*/
@property(nonatomic, readonly) BOOL isEmpty;

/** Возвращает количество ключей в базе данных
 @return Количество ключей
*/
@property(nonatomic, readonly) NSUInteger getKeysCount;

#pragma mark - Key export
/// ****************************************************************************
/// @name Методы экспорта / импорта ключа
/// ****************************************************************************

/**
 Экспорт ключа
 @param keyID Идентификатор ключевой информации в хранилище PayControl
 @param completion block, возвращает параметры в место вызова метода.
 @returns error Ошибка выполнения метода. Если error = nil, то метод выполнился успешно
 @returns keyInfo Экспортированный ключ в XML-формате
 */
- (void)exportKey:(NSString * _Nonnull)keyID completion:(void (^_Nonnull)(PCError * _Nullable error, NSString * _Nullable keyInfo))completion;

#pragma mark - Key import
/** Импорт ключа из объекта QR-кода (PCKeyQRCode).
 @return Экземпляр ключа либо nil в случае ошибки.
 */
- (PCKey * _Nullable)importFromQRCode:(PCQRCode * _Nonnull)qrCode;

/** Импорт ключа из строки с вадидными данными QR-кода ключа.
 @return Экземпляр ключа либо nil в случае ошибки.
 */
- (PCKey * _Nullable)importQRCodeString:(NSString * _Nonnull)qrCodeString;

/**
 Импорт ключа в XML-формате. Handle ключевой информации имеет нулевое значение.
 @param exportedKey Экспортированный ключ в XML-формате
 @return Экземпляр ключа либо nil в случае ошибки.
 */
- (PCKey * _Nullable)importFromXML:(NSString * _Nonnull)xmlString;

/** Получение экзепляра ключа по его идентификатору в хранилище.
 @param keyID строка с индентификатором ключа.
 @return Возвращает экзепляр ключа либо nil в случае ошибки.
 */
- (PCKey * _Nullable)importFromStorage:(NSString * _Nonnull)keyID;

#pragma mark -
/// ****************************************************************************
/// @name Вспомогательные методы
/// ****************************************************************************

/**
 Отпечаток устройства
 @param fingerprintLength Длина отпечатка устройства <= 64 символов (32 байта в виде hex-строки)
 @returns Отпечаток устройства
 */
- (NSString * _Nonnull)getFingerPrint:(NSInteger)fingerprintLength;

/**
 Освобождение ключевой информации всех ключей. Handle ключей принимает нулевое значение
 @returns error ошибка. Если error = nil, то освобождение прошло успешно.
 */
- (PCError * _Nullable)clear;

/**
 Проверка наличия в хранилище ключа с задананным именем.
 @param keyName Имя ключа
 @returns YES если ключ с таким именем есть в хранилище, иначе NO.
 */
- (BOOL)isExistsKeyName:(NSString * _Nonnull)keyName;

/**
 Проверка наличия в хранилище ключа с задананным идентификатором пользователя.
 @param userID Идентификатор пользователя
 @returns YES если ключ с таким идентификатором есть в хранилище, иначе NO.
 */
- (BOOL)isExistsUserID:(NSString * _Nonnull)userID;

/**
 Получение имени ключа из хранилища с задананным идентификатором пользователя.
 @param userID Идентификатор пользователя ключа
 @returns имя ключа, если ключ с таким идентификатором пользователя есть в хранилище, иначе nil.
 */
- (NSString *_Nullable)getKeyNameForUserID:(NSString * _Nonnull)userID;

#pragma mark -
/// ****************************************************************************
/// @name Метод вызова сканера QR-кодов
/// ****************************************************************************
/**
 Запуск сканера.
 @param viewController Контроллер, который отображает контроллер сканера. Контроллер назначается делегатом протокола PCQRCodeReaderDelegate. В нем должны быть реализована методы протокола.
 @param animated  YES - показ сканера с анимацией, NO - без анимации
 @param textToShow Информационный текст
 @param invalidCodeText Текст, который показывается при сканировании неверного QR кода
 @param allowedQRCodeType валидные типы QR кода
 @param maskView cлой-маска отображающийся поверх сканируемого изображения, при этом другие надписи скрываются
 */
- (void)scanQRFromViewController:(UIViewController <PCQRCodeReaderDelegate> * _Nonnull)viewController
                        animated:(BOOL)animated
                      textToShow:(NSString * _Nonnull)textToShow
                 invalidCodeText:(NSString * _Nonnull)invalidCodeText
               allowedQRCodeType:(PCQRCodeAllowedType)allowedType
                        maskView:(UIView * _Nullable)maskView;

- (void)dissmissQRViewController;

@end
